#include <GL/glut.h>
#include <stdio.h>

#include "animal.h"
#include "world.h"
#include "pixel.h"

World::World( int imagec, char** imagev )
{
	image_v = imagev;
	image_c = imagec;
	image_i = 0;
	//find a valid image to base image allocation on
	for ( ; image_i < image_c ; image_i++ )
	{
		image_w = image_width( image_v[image_i] );
		image_h = image_height( image_v[image_i] );

		if ( image_w && image_h ) break;
	}
	if ( !image_w || !image_h ) return; //no valid images found

	//allocate pixels and initialize front buffer to grey.
	image_p = new Pixel*[image_w];
	for ( int i = 0 ; i < image_w ; i++ )
	{
		image_p[i] = new Pixel[image_h];
		for ( int j = 0 ; j < image_h ; j++ )
		{
			image_p[i][j].f_r = 1.0;
			image_p[i][j].f_g = 1.0;
			image_p[i][j].f_b = 1.0;
		}
	}

	//load back buffer from first image
	load_pixels( image_p, image_v[image_i], image_w, image_h );

	//push back to front, load next image
	Promote();
	image_i++;
	if ( image_i >= image_c ) image_i = 0;
	load_pixels( image_p, image_v[image_i], image_w, image_h );

	display_buffer = 'f';

	DNA* dna = new DNA();
	dna->Randomize();
	pop = new Animal( dna );
	pop->x = 50;
	pop->y = 50;
	printf( "velx %f\n", pop->velocity_x );
}

void World::Display()
{
	Step();

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glViewport( 0, 0, screen_w, screen_h );
	glOrtho( world_l, world_r, world_b, world_t, -0.1, 0.1 );
	glMatrixMode( GL_MODELVIEW );

	Pixel p;

	glClearColor( 0, 0, 0, 0 );
	glClear( GL_COLOR_BUFFER_BIT );
	glBegin( GL_POINTS );
	for ( int x = 0 ; x < image_w ; x++ )
	{
		for ( int y = 0 ; y < image_h ; y++ )
		{
			p = image_p[x][y];
			switch ( display_buffer )
			{
			case 'b':
				glColor3f( p.b_r, p.b_g, p.b_b );
				break;
			case 'd':
				glColor3f( p.d_r, p.d_g, p.d_b );
				break;
			default:
			case 'f':
				glColor3f( p.f_r, p.f_g, p.f_b );
				break;
			}	
			glVertex2f( x, y );
		}
	}
	glEnd();

	pop->Render();
}

void World::Resize( int w, int h )
{
	screen_w = w;
	screen_h = h;

	//do these calculations with floats since width/height ratios tend
	//to fall within 1 and 2, even though final product ends up being pixels.
	float sw = float( screen_w );
	float sh = float( screen_h );
	float iw = float( image_w );
	float ih = float( image_h );

	//determine world bounds based on image/screen dimensions

	if ( iw / ih < sw / sh )
	{
		//image is less wide than the screen, so pad world_w
		float ww = ih * sw / sh;
		float dw = ( ww - iw ) / 2.0;
		world_l = int( -dw );
		world_r = int( iw + dw - 1.0 );
		world_b = 0;
		world_t = image_h - 1;
	}
	else if ( iw / ih > sw / sh )
	{
		//image is less tall than the screen, so pad world_h
		float wh = iw * sh / sw;
		float dh = ( wh - ih ) / 2.0;
		world_b = int( -dh );
		world_t = int( ih + dh - 1.0 );
		world_l = 0;
		world_r = image_w - 1;
	}
	else
	{
		//same dimensions, so one-to-one correspondence
		world_l = 0;
		world_r = image_w - 1;
		world_b = 0;
		world_t = image_h - 1;
	}
}

void World::Step()
{
	pop->Step( this );
}

void World::CycleDisplayBuffer()
{
	if ( display_buffer == 'f' ) display_buffer = 'd';
	else if ( display_buffer == 'd' ) display_buffer = 'b';
	else display_buffer = 'f';
}

void World::Promote()
{
	for ( int x = 0 ; x < image_w ; x++ )
	{
		for ( int y = 0 ; y < image_h ; y++ )
		{
			image_p[x][y].f_r = image_p[x][y].b_r;
			image_p[x][y].f_g = image_p[x][y].b_g;
			image_p[x][y].f_b = image_p[x][y].b_b;
			compute_diff( & image_p[x][y] );
		}
	}
}
